;;; Roman/theme.jl
;;; Created by Herman Roozenbeek <hermanr@mailroom.com>

(defgroup Roman-frame "Roman theme options"
  :group appearance)

(defcustom Roman:full-title 'nil
  "Use full width of window for title and buttons"
  :group (appearance Roman-frame)
  :type boolean)

(def-frame-class roman-sticky-button '((cursor . right_ptr))
  (bind-keys roman-sticky-button-keymap 
    "Button1-Click1" 'popup-window-menu
    "Button3-Off" 'toggle-window-sticky))

(let*
  (
    (font default-font)
    (font-colors (list (get-color "#847D7B")
                       "black"))

    (title-width
      (lambda (w)
        (let
          ((w-width (+ 10 (car (window-dimensions w)))))
        (cond ((eq Roman:full-title 'nil)
               (max 0 (min (- w-width 90) (+ 20 (text-width (window-name w) font)))))
              (t
               (max 0 (- w-width 90)))))))

    (transient-title-width
      (lambda (w)
        (let
          ((w-width (+ 10 (car (window-dimensions w)))))
        (cond ((eq Roman:full-title 'nil)
               (max 0 (min (- w-width 50) (+ 20 (text-width (window-name w) font)))))
              (t
               (max 0 (- w-width 50)))))))

    (redraw-this-frame
      (lambda (w)
        (if (eq (window-get w 'current-frame-style) 'Roman) 
          (rebuild-frame w))))

    (rebuild
      (lambda()
        (rebuild-frames-with-style 'Roman)))

    (left-title-images (list (make-image "title-left-inactive.png")
                             (make-image "title-left-active.png")))
    (full-right-title-images (list (make-image "full-title-right-inactive.png")
                              (make-image "full-title-right-active.png")))
    (short-right-title-images (list (make-image "short-title-right-inactive.png")
                              (make-image "short-title-right-active.png")))
    (normal-right-title-images (lambda (w)
                                (if (eq Roman:full-title 'nil)
                                    (if (eq (title-width w) (- (car (window-dimensions w)) 80))
                                        full-right-title-images
                                        short-right-title-images)
                                    full-right-title-images)))
    (transient-right-title-images (lambda (w)
                                (if (eq Roman:full-title 'nil)
                                    (if (eq (title-width w) (- (car (window-dimensions w)) 40))
                                        full-right-title-images
                                        short-right-title-images)
                                    full-right-title-images)))
    (title-images (list (make-image "mid-title-inactive.png")
                        (make-image "mid-title-active.png")))
    (sticky-images (list (make-image "sticky-inactive.png")
                         (make-image "sticky-active.png")))
    (default-icon (make-image "default_icon.png"))
    (icon-image (lambda (w)
                        (if (window-get w 'sticky)
                            sticky-images
                            (let ((icon (window-icon-image w)))
                                 (if (eq icon 'nil) default-icon icon))
                            )))
    (menu-images (list (make-image "menu-inactive.png")
                       (make-image "menu-active.png")))
    (close-images (list (make-image "close-inactive.png") 
                        (make-image "close-active.png")))
    (maximize-images (list (make-image "maximize-inactive.png")
                           (make-image "maximize-active.png")))
    (iconify-images (list (make-image "iconify-inactive.png")
                          (make-image "iconify-active.png")))
    (bottom-image (make-image "bottom-bar.png"))
    (top-image (make-image "top-bar.png"))
    (top-left-image (make-image "top-left.png"))
    (top-right-image (make-image "top-right.png"))
    (bottom-left-image (make-image "bottom-left.png"))
    (bottom-right-image (make-image "bottom-right.png"))
    (left-image (make-image "left-bar.png"))
    (right-image (make-image "right-bar.png"))

    (normal-frame `(
        ((background . ,top-image)
         (top-edge . -5)
         (left-edge . ,(lambda (w) (+ (title-width w) 80)))
         (right-edge . 0)
         (class . title))
        ((background . ,left-image)
         (left-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (class . left-border))
        ((background . ,top-right-image)
         (right-edge . -5)
         (top-edge . -5)
         (class . title))
        ((background . ,right-image)
         (right-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (class . right-border))
        ((background . ,bottom-image)
         (left-edge . 0)
         (right-edge . 0)
         (bottom-edge . -5)
         (class . bottom-border))
        ((background . ,bottom-left-image)
         (left-edge . -5)
         (bottom-edge . -5)
         (class . bottom-left-corner))
        ((background . ,bottom-right-image)
         (right-edge . -5)
         (bottom-edge . -5)
         (class . bottom-right-corner))
        ((background . ,title-images)
         (foreground . ,font-colors)
         (font . ,font)
         (text . ,window-name)
         (top-edge . -20)
         (left-edge . 20)
         (width . ,title-width)
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,left-title-images)
         (left-edge . -5)
         (top-edge . -20)
         (class . title))
        ((background . ,normal-right-title-images)
         (right-edge . ,(lambda (w) (- (car (window-dimensions w)) 85 (title-width w))))
         (top-edge . -20)
         (class . title))
        ((background . ,menu-images)
         (left-edge . 0)
         (top-edge . -20)
         (class . roman-sticky-button))
        ((background . ,icon-image)
         (left-edge . 0)
         (top-edge . -18)
         (width . 16)
         (height . 16)
         (class . roman-sticky-button))
        ((background . ,iconify-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 20)))
         (top-edge . -20)
         (class . iconify-button))
        ((background . ,maximize-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 40)))
         (top-edge . -20)
         (class . maximize-button))
        ((background . ,close-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 60)))
         (top-edge . -20)
         (class . close-button))
      ))
    (transient-normal-frame `(
        ((background . ,top-image)
         (top-edge . -5)
         (left-edge . ,(lambda (w) (+ (title-width w) 40)))
         (right-edge . 0)
         (class . title))
        ((background . ,left-image)
         (left-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (class . left-border))
        ((background . ,top-right-image)
         (right-edge . -5)
         (top-edge . -5)
         (class . title))
        ((background . ,right-image)
         (right-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (class . right-border))
        ((background . ,bottom-image)
         (left-edge . 0)
         (right-edge . 0)
         (bottom-edge . -5)
         (class . bottom-border))
        ((background . ,bottom-left-image)
         (left-edge . -5)
         (bottom-edge . -5)
         (class . bottom-left-corner))
        ((background . ,bottom-right-image)
         (right-edge . -5)
         (bottom-edge . -5)
         (class . bottom-right-corner))
        ((background . ,title-images)
         (foreground . ,font-colors)
         (font . ,font)
         (text . ,window-name)
         (top-edge . -20)
         (left-edge . 20)
         (width . ,transient-title-width)
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,left-title-images)
         (left-edge . -5)
         (top-edge . -20)
         (class . title))
        ((background . ,transient-right-title-images)
         (right-edge . ,(lambda (w) (- (car (window-dimensions w)) 45 (transient-title-width w))))
         (top-edge . -20)
         (class . title))
        ((background . ,menu-images)
         (left-edge . 0)
         (top-edge . -20)
         (class . roman-sticky-button))
        ((background . ,icon-image)
         (left-edge . 0)
         (top-edge . -18)
         (width . 16)
         (height . 16)
         (class . roman-sticky-button))
        ((background . ,close-images)
         (left-edge . ,(lambda (w) (+ (transient-title-width w) 20)))
         (top-edge . -20)
         (class . close-button))
      ))
    (shaded-frame `(
        ((background . ,title-images)
         (foreground . ,font-colors)
         (font . ,font)
         (text . ,window-name)
         (top-edge . -20)
         (left-edge . 20)
         (width . ,title-width)
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,left-title-images)
         (left-edge . -5)
         (top-edge . -20)
         (class . title))
        ((background . ,full-right-title-images)
         (right-edge . ,(lambda (w) (- (car (window-dimensions w)) 85 (title-width w))))
         (top-edge . -20)
         (class . title))
        ((background . ,menu-images)
         (left-edge . 0)
         (top-edge . -20)
         (class . roman-sticky-button))
        ((background . ,icon-image)
         (left-edge . 0)
         (top-edge . -18)
         (width . 16)
         (height . 16)
         (class . roman-sticky-button))
        ((background . ,iconify-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 20)))
         (top-edge . -20)
         (class . iconify-button))
        ((background . ,maximize-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 40)))
         (top-edge . -20)
         (class . maximize-button))
        ((background . ,close-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 60)))
         (top-edge . -20)
         (class . close-button))
        ((background . ,bottom-image)
         (left-edge . 0)
         (top-edge . 0)
         (width . ,(lambda (w) (+ (title-width w) 80)))
         (class . title))
        ((background . ,bottom-left-image)
         (left-edge . -5)
         (top-edge . 0)
         (class . title))
        ((background . ,bottom-right-image)
         (right-edge . ,(lambda (w) (- (car (window-dimensions w)) 85 (title-width w))))
         (top-edge . 0)
         (class . title))
      ))
    (transient-shaded-frame `(
        ((background . ,title-images)
         (foreground . ,font-colors)
         (font . ,font)
         (text . ,window-name)
         (top-edge . -20)
         (left-edge . 20)
         (width . ,transient-title-width)
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,left-title-images)
         (left-edge . -5)
         (top-edge . -20)
         (class . title))
        ((background . ,full-right-title-images)
         (right-edge . ,(lambda (w) (- (car (window-dimensions w)) 45 (transient-title-width w))))
         (top-edge . -20)
         (class . title))
        ((background . ,menu-images)
         (left-edge . 0)
         (top-edge . -20)
         (class . roman-sticky-button))
        ((background . ,icon-image)
         (left-edge . 0)
         (top-edge . -18)
         (width . 16)
         (height . 16)
         (class . roman-sticky-button))
        ((background . ,close-images)
         (left-edge . ,(lambda (w) (+ (transient-title-width w) 20)))
         (top-edge . -20)
         (class . close-button))
        ((background . ,bottom-image)
         (left-edge . 0)
         (top-edge . 0)
         (width . ,(lambda (w) (+ (transient-title-width w) 40)))
         (class . title))
        ((background . ,bottom-left-image)
         (left-edge . -5)
         (top-edge . 0)
         (class . title))
        ((background . ,bottom-right-image)
         (right-edge . ,(lambda (w) (- (car (window-dimensions w)) 45 (transient-title-width w))))
         (top-edge . 0)
         (class . title))
      ))
    )

  (add-frame-style 'Roman
    (lambda (w type)
      (case type 
            ((default) normal-frame)
            ((shaded) shaded-frame)
            ((transient) transient-normal-frame)
            ((shaded-transient) transient-shaded-frame))))
            
  (mapc (lambda (sym)
         (custom-set-property sym ':after-set rebuild))
        '(Roman:full-title))

  (call-after-property-changed 'WM_NAME redraw-this-frame)
  (call-after-property-changed 'WM_HINTS redraw-this-frame)
  (call-after-state-changed '(sticky) redraw-this-frame)
)
